
/*************************************************************************** 
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "DisplayManager.hpp"

#include "ResourcePool.hpp"
#include "Game.hpp"

#include "graphlib/GLFont.hpp"

// --------------------------------------------------------

static const char *menuTitle[] = {	"New Game",
									"Load Game",
									"Options",
									"Credits",
									"Quit"
								};
static int menuCount = 5;

// --------------------------------------------------------

/**
 * Default constructor
 */
DisplayManager::DisplayManager(void)
{
	font = new GLFont("./data/images/FontForte.bmp", 24, 32, 20, 4);
}

// --------------------------------------------------------

/**
 * destructor
 */
DisplayManager::~DisplayManager()
{
	delete font;
}

// --------------------------------------------------------

/**
 * Initialization of display functions
 */
void DisplayManager::initDisplay(void)
{
	// SDL iinitialization
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) <0)
	{
		  fprintf(stderr,"ERROR : SDL Initialization : %s\n",SDL_GetError());
		  exit(1);
	}
	spritesCount = 0;
}

// --------------------------------------------------------

/**
 * reinitialization of display functions.
 */
void DisplayManager::closeDisplay(void)
{
	SDL_Quit();
}

// --------------------------------------------------------

/**
 * create screen with wanted video mode.
 */
void DisplayManager::createScreen(int width, int height, int depth, bool fullScreen)
{
	screen = SDL_SetVideoMode(width,height,depth,SDL_HWSURFACE|SDL_DOUBLEBUF|((fullScreen)?SDL_FULLSCREEN:0));
	if(screen == NULL)
	{   
		fprintf(stderr,"ERROR : SDL - Unable to set video mode : %s\n", SDL_GetError()); 
		exit(1); 
	}
	controlKeys = ResourcePool::defaultPool.loadImage("controlKeys");
	controlLifes = ResourcePool::defaultPool.loadImage("controlLifes");

	screenSize.set(width,height);
}

// --------------------------------------------------------

/**
 * Initialize the screen with the background.
 */
void DisplayManager::initView(void)
{
	SDL_BlitSurface(background,NULL,screen,NULL);
}

// --------------------------------------------------------

/**
 * Create the background of all the game...
 *
 * @todo : transfer the background into the level environment.
 */
void DisplayManager::createBackground(char* name)
{
	// surface to flip
	background = ResourcePool::defaultPool.loadImage(name);
	background = SDL_DisplayFormat(background);

	if(background == NULL)
	{
		fprintf(stderr,"ERROR : Unable to create background : %s\n", SDL_GetError()); 
		exit(1); 
	}
}

// --------------------------------------------------------

/**
 * Add a sprite to the screen.
 */
void DisplayManager::addSprite(SDL_Surface* sprite, int x, int y, int column, int row)
{
	SDL_Rect destinationRect;
	destinationRect.y = y;
	destinationRect.x = x;
	destinationRect.w = _TAILLESPRITEPIX;
	destinationRect.h = _TAILLESPRITEPIX;

	SDL_Rect sourceRect;
	sourceRect.y = _TAILLESPRITEPIX*row;
	sourceRect.x = _TAILLESPRITEPIX*column;
	sourceRect.w = _TAILLESPRITEPIX;
	sourceRect.h = _TAILLESPRITEPIX;

	SDL_SetColorKey(sprite,SDL_SRCCOLORKEY,SDL_MapRGB(sprite->format, 0, 0, 255));
	SDL_BlitSurface(sprite,&sourceRect,screen,&destinationRect);
}
	
// --------------------------------------------------------

/**
 * Add a sprite to the screen.
 * same as previous but affects backsave
 *
 * @ todo: verify : this method is exactly the same than AddSprite without column choice.
 */
void DisplayManager::addMapSprite(SDL_Surface* sprite, int x, int y, int column)
{
	SDL_Rect destinationRect;
	destinationRect.y = y;
	destinationRect.x = x;
	destinationRect.w = _TAILLESPRITEPIX;
	destinationRect.h = _TAILLESPRITEPIX;

	SDL_Rect sourceRect;
	sourceRect.y = 0;
	sourceRect.x = _TAILLESPRITEPIX*column;
	sourceRect.w = _TAILLESPRITEPIX;
	sourceRect.h = _TAILLESPRITEPIX;

	SDL_SetColorKey(sprite,SDL_SRCCOLORKEY,SDL_MapRGB(sprite->format, 0, 0, 255));
	SDL_BlitSurface(sprite,&sourceRect,screen,&destinationRect);
}

// --------------------------------------------------------

/**
 * update entire screen...
 */
void DisplayManager::refreshScreen()
{
	SDL_Flip(screen);
}

// --------------------------------------------------------

/**
 * Display the introduction screen of one level
 *
 * @todo : transfer this method to level environment.
 * @todo : delay should can be interrupted...
 */
void DisplayManager::displayLevelIntro(const char *name)
{
	SDL_Surface *back = ResourcePool::defaultPool.loadImage("flatback");
	SDL_Surface *bibom = ResourcePool::defaultPool.loadImage("bibomBig");
	SDL_Rect alignRect;
	int x;

	alignRect.x = (screenSize.x - back->w) / 2;
	alignRect.y = (screenSize.y - back->h) / 2;
	SDL_BlitSurface(back,NULL,screen,&alignRect);

	alignRect.x = (screenSize.x - bibom->w) / 2;
	alignRect.y = 10;
	SDL_SetColorKey(bibom,SDL_SRCCOLORKEY,SDL_MapRGB(bibom->format, 0, 0, 255));
	SDL_BlitSurface(bibom, NULL, screen, &alignRect);

	x = (screenSize.x - font->getTextSize(name).x) / 2;
	font->writeText(GLPoint(x, 150), name, screen);

	SDL_Flip(screen);
}

// --------------------------------------------------------

/**
 * Display game info on the game screen.
 */
void DisplayManager::displayControlBar(int keysCount, int lifeCount)
{
	char string[10];

	SDL_Rect keysIconDSTrect;
	keysIconDSTrect.y = screenSize.y - _TAILLESPRITEPIX;
	keysIconDSTrect.x = 0;

	SDL_Rect lifesIconDSTrect;
	lifesIconDSTrect.y = screenSize.y - _TAILLESPRITEPIX;
	lifesIconDSTrect.x = screenSize.x - controlLifes->w;

	// --- icons
	SDL_SetColorKey(controlKeys,SDL_SRCCOLORKEY,SDL_MapRGB(controlKeys->format, 0, 0, 255));
	SDL_BlitSurface(controlKeys,NULL,screen,&keysIconDSTrect);
	SDL_SetColorKey(controlLifes,SDL_SRCCOLORKEY,SDL_MapRGB(controlLifes->format, 0, 0, 255));
	SDL_BlitSurface(controlLifes,NULL,screen,&lifesIconDSTrect);

	// --- numbers
	sprintf(string, "%d", keysCount);
	font->writeText(GLPoint(0,screenSize.y - _TAILLESPRITEPIX), string, screen);
	sprintf(string, "%d", lifeCount);
	font->writeText(GLPoint(screenSize.x - font->getLetterSize().x, screenSize.y - _TAILLESPRITEPIX), string, screen);
}	
	
// --------------------------------------------------------

/**
 * display the game over screen...
 */
void DisplayManager::displayGameOver(void)
{
	SDL_Surface *back = ResourcePool::defaultPool.loadImage("flatback");
	SDL_Surface *bibom = ResourcePool::defaultPool.loadImage("bibomBig");
	SDL_Rect alignRect;
	int x;

	alignRect.x = (screenSize.x - back->w) / 2;
	alignRect.y = (screenSize.y - back->h) / 2;
	SDL_BlitSurface(back,NULL,screen,&alignRect);

	alignRect.x = (screenSize.x - bibom->w) / 2;
	alignRect.y = 10;
	SDL_SetColorKey(bibom,SDL_SRCCOLORKEY,SDL_MapRGB(bibom->format, 0, 0, 255));
	SDL_BlitSurface(bibom, NULL, screen, &alignRect);

	x = (screenSize.x - font->getTextSize("GAME OVER").x) / 2;
	font->writeText(GLPoint(x, 150), "GAME OVER", screen);

	SDL_Flip(screen);
}

// --------------------------------------------------------

/**
 * display the menu screen. 
 *
 * @todo : this screen should be an interactive menu. it's control should be into Game class.
 */
void DisplayManager::displayMenu(EMenuChoice item)
{
	SDL_Surface *back = ResourcePool::defaultPool.loadImage("flatback");
	SDL_Surface *bibom = ResourcePool::defaultPool.loadImage("bibomBig");
	SDL_Surface *title = ResourcePool::defaultPool.loadImage("title");
	SDL_Surface *pointer = ResourcePool::defaultPool.loadImage("MenuPointer");
	SDL_Rect alignRect;
	int menuY, menuX, menuWidth;
	GLPoint size;
	int i;

	alignRect.x = (screenSize.x - back->w) / 2;
	alignRect.y = (screenSize.y - back->h) / 2;
	SDL_BlitSurface(back,NULL,screen,&alignRect);

	alignRect.x = (screenSize.x - bibom->w) / 2;
	alignRect.y = 10;
	SDL_SetColorKey(bibom,SDL_SRCCOLORKEY,SDL_MapRGB(bibom->format, 0, 0, 255));
	SDL_BlitSurface(bibom, NULL, screen, &alignRect);

	alignRect.x = (screenSize.x - title->w) / 2;
	alignRect.y = 10 + bibom->h + 10;
	SDL_BlitSurface(title, NULL, screen, &alignRect);

	menuY = 30 + bibom->h + title->h;
	menuWidth = 0;
	for(i=0; i<menuCount; i++)
	{
		size = font->getTextSize(menuTitle[i]);
		if(size.x > menuWidth)
			menuWidth = size.x;
	}

	menuX = _TAILLESPRITEPIX + 20 + (screenSize.x - (_TAILLESPRITEPIX + 20 + menuWidth)) / 2;
	for(i=0; i<menuCount; i++)
	{
		font->writeText(GLPoint(menuX, menuY+i*font->getLetterSize().y), menuTitle[i], screen);
	}

	addSprite(pointer, menuX - _TAILLESPRITEPIX - 20, menuY+item*font->getLetterSize().y, 0, 0);
	
	SDL_Flip(screen);
}	

// --------------------------------------------------------

/**
 * display the credits screen. 
 *
 * @todo : this screen should be an interactive menu. it's control should be into Game class.
 */
void DisplayManager::displayCredits(void)
{
	SDL_Surface *back= ResourcePool::defaultPool.loadImage("back");
	SDL_Rect alignRect;
	alignRect.x = (screenSize.x - back->w) / 2;
	alignRect.y = (screenSize.y - back->h) / 2;
	int x;

	SDL_BlitSurface(back,NULL,screen,&alignRect);

	x = (screenSize.x - font->getTextSize("idea").x) / 2;
	font->writeText(GLPoint(x, 50), "idea", screen);

	x = (screenSize.x - font->getTextSize("Vincent PELLETIER").x) / 2;
	font->writeText(GLPoint(x, 100), "Vincent PELLETIER", screen);

	x = (screenSize.x - font->getTextSize("coding").x) / 2;
	font->writeText(GLPoint(x, 200), "coding", screen);

	x = (screenSize.x - font->getTextSize("Vincent PELLETIER").x) / 2;
	font->writeText(GLPoint(x, 250), "Vincent PELLETIER", screen);

	x = (screenSize.x - font->getTextSize("Sbastien DONCKER").x) / 2;
	font->writeText(GLPoint(x, 282), "Sbastien DONCKER", screen);

	x = (screenSize.x - font->getTextSize("art").x) / 2;
	font->writeText(GLPoint(x, 382), "art", screen);

	x = (screenSize.x - font->getTextSize("Vincent PELLETIER").x) / 2;
	font->writeText(GLPoint(x, 432), "Vincent PELLETIER", screen);

	SDL_Flip(screen);
}	

// --------------------------------------------------------

