
/*************************************************************************** 
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "Environment.hpp"

#include "Hero.hpp"
#include "Enemy.hpp"
#include "MapManager.hpp"
#include "DisplayManager.hpp"
#include "ResourcePool.hpp"

// --------------------------------------------------------

Environment::Environment(DisplayManager* aDisplay, MapManager* aMap)
: destroyedBlocks(true)
{
	display = aDisplay;
	map = aMap;
	
	// create characters
	createHero();
	createEnemies();

	// load map elem surface
	mapFloor 		= ResourcePool::defaultPool.loadImage("floor");
	mapScale 		= ResourcePool::defaultPool.loadImage("scale");
	mapScaleDown 	= ResourcePool::defaultPool.loadImage("scale_down");
	mapScaleUp 		= ResourcePool::defaultPool.loadImage("scale");
	mapDoorE 		= ResourcePool::defaultPool.loadImage("door_e");
	mapDoorS 		= ResourcePool::defaultPool.loadImage("door_s");
	mapKey 			= ResourcePool::defaultPool.loadImage("key");
}

// --------------------------------------------------------

Environment::~Environment()
{
}

// --------------------------------------------------------

void Environment::newLevel(void)
{
	// map initialization
	mapSize = map->getSize() * _MAP_BLOCK_SIZE;

	mapOffset.set((mapSize.x-display->getSize().x) / 2, (mapSize.y-display->getSize().y) / 2);
	newMapOffset = mapOffset;

	motionLessScreen.x = _MOTIONLESS_SCREEN_MARGIN_W;
	motionLessScreen.y = _MOTIONLESS_SCREEN_MARGIN_H;
	motionLessScreen.w = display->getSize().x - 2 * _MOTIONLESS_SCREEN_MARGIN_W;
	motionLessScreen.h = display->getSize().y - 2 * _MOTIONLESS_SCREEN_MARGIN_H;

	// characters initialization
	hero->newLevel();
	enemy[0]->newLevel();
}

// --------------------------------------------------------

void Environment::refreshView(void)
{
 	// init screen with background
	display->initView();
	// add map elem
 	displayMap();

	// display control bar
 	display->displayControlBar(hero->getKeysCount(),hero->getLifeCount());

	// put characters into screen
	hero->display(display, mapOffset);
	enemy[0]->display(display, mapOffset);

	// refresh
 	display->refreshScreen();		
}

// --------------------------------------------------------

void Environment::evolve(void)
{
	bool turn = true;

	do
	{
		turn &= makeEnvironmentEvolve();
		turn &= makeCharactersEvolve();

		refreshView();
		SDL_Delay(5);

	} while(turn);
}

// --------------------------------------------------------

bool Environment::makeEnvironmentEvolve(void)
{
	if(mapOffset.x > newMapOffset.x)
	{
		if(_MAP_SCROLLING_SPEED <= mapOffset.x - newMapOffset.x)
			mapOffset.x -= _MAP_SCROLLING_SPEED;
		else mapOffset.x = newMapOffset.x;
	}
	else if(mapOffset.x < newMapOffset.x)
	{
		if(_MAP_SCROLLING_SPEED <= newMapOffset.x - mapOffset.x)
			mapOffset.x += _MAP_SCROLLING_SPEED;
		else mapOffset.x = newMapOffset.x;
	}
	if(mapOffset.y > newMapOffset.y)
	{
		if(_MAP_SCROLLING_SPEED <= mapOffset.y - newMapOffset.y)
			mapOffset.y -= _MAP_SCROLLING_SPEED;
		else mapOffset.y = newMapOffset.y;
	}
	else if(mapOffset.y < newMapOffset.y)
	{
		if(_MAP_SCROLLING_SPEED <= newMapOffset.y - mapOffset.y)
			mapOffset.y += _MAP_SCROLLING_SPEED;
		else mapOffset.y = newMapOffset.y;
	}

	updateTimeDestroyedBlock();
	return true;
}

// --------------------------------------------------------

bool Environment::makeCharactersEvolve(void)
{
	if (hero->isMoveWaiting())
	{
		hero->evolve();
	}
		
	enemy[0]->move();
	if (enemy[0]->isMoveWaiting())
		enemy[0]->evolve();

	// collisions...
	if(hero->getPosition() == enemy[0]->getPosition())
		hero->collide();

	return ((hero->isMoveWaiting()) && (enemy[0]->isMoveWaiting()));
}

// --------------------------------------------------------

void Environment::displayMap(void)
{
	if (map == NULL)
	{
		  fprintf(stderr,"ERROR : no map loaded.\n"); 
		  exit(1); 
	}

	for (int i = 0;i < map->getMapWidth();i++)
	{
		for (int j = 0;j < map->getMapHeight();j++)
		{
			char curr = map->getElement(i,j);

			switch (curr)
			{
				case 'f':
					display->addMapSprite(mapFloor,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				case 's':
					display->addMapSprite(mapScale,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				case 'b':
					display->addMapSprite(mapDoorE,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				case 'e':
					display->addMapSprite(mapDoorS,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				case 'k':
					display->addMapSprite(mapKey,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				case 'u':
					display->addMapSprite(mapScaleUp,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				case 'd':
					display->addMapSprite(mapScaleDown,i*_TAILLESPRITEPIX - mapOffset.x, j*_TAILLESPRITEPIX - mapOffset.y, 0);
					break;
				default:
					break;
			}
		}
	}
}

// --------------------------------------------------------

int Environment::levelFinished(void)
{
	if(hero->isDead())
		return 2;
	if(hero->finishLevel())
		return 1;
	else
		return 0;
}


// --------------------------------------------------------


void Environment::destroyBlock(int x, int y)
{
	DestroyedBlock *block;

	// verify this block is not already destroyed...
	for (int j=0; j<destroyedBlocks.getCount(); j++)
	{
		block = (DestroyedBlock *)destroyedBlocks[j];
		if((block->position.x == x) && (block->position.y == y))
		{
			block->timeElapsed = 0;
			return;
		}
	}

	block =new DestroyedBlock;
	block->position.set(x, y);
	block->timeElapsed = 0;
	destroyedBlocks.add(block);

	map->setElement(x, y, btBlank);
}

// --------------------------------------------------------

void Environment::updateTimeDestroyedBlock(void)
{
	int i=0;
	DestroyedBlock *block;
	
	while(i < destroyedBlocks.getCount())
	{
		block = (DestroyedBlock *)destroyedBlocks[i];
		if (block->timeElapsed >= _TIME_RECOVERY_BLOCK)
		{
			map->setElement(block->position.x, block->position.y, btGround);

			// crush test
			if(hero->getPosition() == block->position)
				hero->crush();
			if(enemy[0]->getPosition() == block->position)
				enemy[0]->crush();
			
			
			// WARNING : this change the count of this loop. This is a terrible bug prone code !!! be carefull
			destroyedBlocks.remove(i);
		}
		else
		{
			block->timeElapsed++;
			i++;
		}
	}
}

// --------------------------------------------------------

void Environment::createHero(void)
{
	SDL_Surface *hero_sprite = ResourcePool::defaultPool.loadImage("explorateurSprite");
	hero = new Hero(this, map,hero_sprite,4,3);
}

// --------------------------------------------------------

void Environment::createEnemies(void)
{
	SDL_Surface *enemy_sprite = ResourcePool::defaultPool.loadImage("enemySprite");
	enemy = new Enemy* [_MAXENEMY];	
	enemy[0] = new Enemy(this, map,enemy_sprite,hero,2);
}

// --------------------------------------------------------

void Environment::setMapOffset(const GLPoint& offset)
{
	newMapOffset = offset;
}

// --------------------------------------------------------

void Environment::setMapOffset(int x, int y)
{
	newMapOffset.set(x, y);
}

// --------------------------------------------------------

const GLPoint& Environment::getScreenSize(void) const
{
	return display->getSize();
}

// --------------------------------------------------------
