
/*************************************************************************** 
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "Game.hpp"

#include "ResourcePool.hpp"
#include "OptionsManager.hpp"
#include "Hero.hpp"

#include <string.h>

// --------------------------------------------------------

// --------------------------------------------------------

/**
 * Default constructor
 *
 * @todo : video mode from options.
 */
Game::Game(void)
{
	OptionsManager::VideoMode vMode;

	// create screen
	display = new DisplayManager();
	display->initDisplay();

	vMode = OptionsManager::defaultManager.getVideoMode();
	display->createScreen(vMode.width, vMode.height, vMode.depth, (vMode.mode == OptionsManager::wmFullScreen));

	// map
	map = new MapManager(OptionsManager::defaultManager.getFolders().maps);

	// menu phase...
	menuChoice = mcNew;
}

// --------------------------------------------------------

/**
 * destructor
 */
Game::~Game()
{
	if(display != NULL)
		delete display;
	if(map != NULL)
		delete map;

	SDL_Quit();
}

// --------------------------------------------------------

/**
 * Main method. Run all the game.
 *
 */
void Game::runGame(void)
{
	const char *mapName;
	int i=0;
	int levelResult = 1;

	// load background
	display->createBackground("back");
	Environment* env = new Environment(display,map);

	mapName = OptionsManager::defaultManager.getGameParameters().startLevel;

	while((levelResult == 1) && (mapName != NULL) && (strcmp(mapName,"END") != 0))
	{
		map->loadMap(mapName);
		// refresh screen
		display->initView();
		// display level intro
		display->displayLevelIntro(map->getName());
		SDL_Delay(1000);
		// load background file
		display->createBackground("back");
		// init hero for new level
		env->newLevel();
		// print map, print hero and refresh screen
		env->refreshView();

		levelResult = runLevel(env);

		// next level.
		i++;
		mapName = map->getNextMapName();
	}

	display->displayGameOver();
	SDL_Delay(2000);
	clearEvents();
}

// --------------------------------------------------------

/**
 * Run a level with a particular environment.
 * @return 0 if player dead, 1 if next level, 2 if quit
 * 
 * @todo : runLevel can be transferred to Environment class....
 */
int Game::runLevel(Environment* env)
{
	Uint8 *keystate;

	while(!env->levelFinished())
	{
		SDL_PumpEvents(); // feed the event buffer
		keystate = SDL_GetKeyState(NULL); // feed the keystate array

		if (keystate[SDLK_ESCAPE])
		{
			return 2;
		}
		if (keystate[SDLK_LEFT])
		{
			env->getHero()->moveLeft();
		}
		else if (keystate[SDLK_RIGHT])
		{
			env->getHero()->moveRight();
		}
		else if (keystate[SDLK_UP])
		{
			env->getHero()->moveUp();
		}
		else if (keystate[SDLK_DOWN])
		{
			env->getHero()->moveDown();
		}
		if (keystate[SDLK_q])
		{
			env->getHero()->putHoleLeft();
		}
		if (keystate[SDLK_d])
		{
			env->getHero()->putHoleRight();
		}
		if (keystate[SDLK_n])
		{
			return 1;
		}

		env->evolve();		

		//SDL_Delay(5);
	}
	
	// return the finish statement
	return env->levelFinished();
}

// --------------------------------------------------------

void Game::runMenu(void)
{
	bool end = false;
	bool changesOccured;
	SDL_Event event;


	do
	{
		display->displayMenu(menuChoice);

		/*
		SDL_PumpEvents(); 
		keystate = SDL_GetKeyState(NULL);
		*/
		changesOccured = false;
		do
		{
			SDL_WaitEvent(&event);

			if(event.type == SDL_KEYDOWN)
			{
				switch(event.key.keysym.sym)
				{
					case SDLK_ESCAPE:
						end = true;
						changesOccured = true;
						break;
					case SDLK_UP:
						if(menuChoice == mcNew)
							menuChoice = mcQuit;
						else
							menuChoice = (EMenuChoice)((int)menuChoice - 1);
						changesOccured = true;
						break;
					case SDLK_DOWN:
						if(menuChoice == mcQuit)
							menuChoice = mcNew;
						else
							menuChoice = (EMenuChoice)((int)menuChoice + 1);
						changesOccured = true;
						break;
					case SDLK_RETURN:
					case SDLK_KP_ENTER:
						switch(menuChoice)
						{
							case mcNew:
								runGame();
								break;
							case mcLoad:
								printf("[error] Load Menu Item not implemented yet.\n");
								break;
							case mcOptions:
								printf("[error] Options Menu Item not implemented yet.\n");
								break;
							case mcCredits:
								credits();
								break;
							case mcQuit:
								end = true;
								break;
						}
						changesOccured = true;
						break;
				};
			}

		} while(!changesOccured);

	} while(!end);
	
}

// --------------------------------------------------------

void Game::credits(void)
{
	SDL_Event event;

	display->displayCredits();

	SDL_Delay(1000);
	clearEvents();
	do
	{
		SDL_WaitEvent(&event);
	}while(event.type != SDL_KEYUP);
}

// --------------------------------------------------------

void Game::run(void)
{
	runMenu();

	display->displayCredits();
	SDL_Delay(_DELAY_CREDITS);
}

// --------------------------------------------------------

void Game::clearEvents(void)
{
	SDL_Event event;

	while(SDL_PollEvent(&event));
}

// --------------------------------------------------------
