
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "MapManager.hpp"

#include "toollib/TLFiles.hpp"

#include <string.h>
#include <io.h>
#include <fcntl.h>

// --------------------------------------------------------

MapManager::MapManager(const char *mapsPath)
: size(_MAPW, _MAPH)
{
	map = NULL;
	path = NULL;
	nextMapName = NULL;
	name = NULL;

	setPath(mapsPath);
}

// --------------------------------------------------------

MapManager::~MapManager()
{
	if(path != NULL)
		delete[] path;

	if(nextMapName != NULL)
		delete[] nextMapName;

	if(name != NULL)
		delete[] name;
}

// --------------------------------------------------------

void MapManager::loadMap(const char *mapName)
{
	int mapFile;
	char *fileName;
	int fileNameLength;
	int ret, len, i, j;
	char line[1024];
	char byte;

	// --- initializations.
	keysCount = 0;

	// --- open the file.
	if(path == NULL)
		fileNameLength = strlen(mapName)+strlen(_MAPFILES_SUFFIX);
	else
		fileNameLength = strlen(path)+strlen(mapName)+strlen(_MAPFILES_SUFFIX);
	fileName = new char[fileNameLength+1];
	if(path != NULL)
	{
		strcpy(fileName, path);
		strcat(fileName, mapName);
	}
	else
		strcpy(fileName, mapName);
	strcat(fileName, _MAPFILES_SUFFIX);

	mapFile = open(fileName, O_RDONLY|O_TEXT);
	if(mapFile < 0)
	{
		fprintf(stderr,"ERROR : Unable to load the map <%s>\n", fileName); 
		exit(1); 
	}

	// --- get the name of the map
	if(TLFiles::readLine(mapFile, line, 1024))
	{
		if(name != NULL)
		{
			delete[] name;
			name = NULL;
		}
		len = strlen(line);
		if(len > 0)
		{
			name = new char[len+1];
			strcpy(name, line);
		}
printf("map name = <%s>\n", name);
	}
	// --- get the name of the next map file.
	if(TLFiles::readLine(mapFile, line, 1024))
	{
		if(nextMapName != NULL)
		{
			delete[] nextMapName;
			nextMapName = NULL;
		}
		len = strlen(line);
		if(len > 0)
		{
			nextMapName = new char[len+1];
			strcpy(nextMapName, line);
		}
printf("next map name = <%s>\n", nextMapName);
	}

	// --- destroy old map.
	if (map != NULL)
	{
		for(int i=0; i<getMapHeight(); i++)
			delete[] map[i];
		delete[] map;
	}

	// --- width of map
	if(TLFiles::readLine(mapFile, line, 1024))
	{
		size.x = atoi(line);
printf("map width = %d\n", size.x);
	}

	// --- height of map
	if(TLFiles::readLine(mapFile, line, 1024))
	{
		size.y = atoi(line);
printf("map height = %d\n", size.y);
	}

	// --- verifications....
	if((size.x <= 0) || (size.y <= 0))
	{
		fprintf(stderr, "ERROR : The map <%s> has a bad size.\n", fileName);
		exit(1);
	}

	// --- create map array.
	map = new EBlockType *[getMapHeight()];
	for (i=0;i<getMapHeight();i++)
	{
		map[i] = new EBlockType[getMapWidth()];
	}

	// --- get the map datas.
	for (i=0;i<getMapHeight();i++)
	{
		for (j=0;j < getMapWidth();j++)
		{
			ret = read(mapFile,&byte, sizeof(char));
			if(ret <= 0)
			{
				fprintf(stderr,"ERROR : The map <%s> has a bad format.\n", fileName);
				exit(1); 
			}
			map[i][j] = (EBlockType)((int)byte);

			if (map[i][j] == btEntry) // check if this is the entrypoint
			{
				entryPoint.x = j;
				entryPoint.y = i;
			}
			if (map[i][j] == btExit) // check if this is the exitpoint
			{
				exitPoint.x = j;
				exitPoint.y = i;
			}
			if (map[i][j] == btKey) // check if this is a key
			{
				keysCount++;
			}
			if (map[i][j] == btEnemyEntry) // check if this enemy entry point
			{
				enemyEntryPoint.x = j;
				enemyEntryPoint.y = i;
			}
		}
		// this is for eat the EOL
		do
		{
			ret = read(mapFile,&byte, sizeof(char));
		} while((ret > 0 ) && (byte != '\n') && (byte != '\r'));
	}

	// --- close the file. Terminate initialization.
	close (mapFile);
}

// --------------------------------------------------------

void MapManager::setPath(const char *mapsPath)
{
	bool addSlash;
	int len;

	if(path != NULL)
	{
		delete[] path;
		path = NULL;
	}
	if(mapsPath != NULL)
	{
		len = strlen(mapsPath);
		addSlash = ((mapsPath[len - 1] != '\\') && (mapsPath[len - 1] != '/'));
				
		path = new char[len+1+(addSlash?1:0)];
		strcpy(path, mapsPath);
		if(addSlash)
			strcat(path, "/");
	}
}

// --------------------------------------------------------

void MapManager::setElement(int x, int y, EBlockType element)
{
	if(isInMap(x,y))
	{
		map[y][x] = element;
	}
}

// --------------------------------------------------------

void MapManager::setElement(const GLPoint& position, EBlockType element)
{
	setElement(position.x, position.y, element);
}

// --------------------------------------------------------
