
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _MAPMANAGER_
#define _MAPMANAGER_

#include "DisplayManager.hpp"

#include "graphlib/GLPoint.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>

// --------------------------------------------------------

#define _MAPW					  25
#define _MAPH					  18

#define _MAPFILES_SUFFIX		".map"

// size of square blocks of map.
#define _MAP_BLOCK_SIZE			  32	

// --------------------------------------------------------

typedef enum {	btOutOfMap = -1,
				btBlank = '#', 
				btEntry = 'b', 
				btEnemyEntry = 'x',
				btExit = 'e', 
				btGround = 'f', 
				btScale = 's',
				btScaleUp = 'u',
				btScaleDown = 'd',
				btKey = 'k'
			} EBlockType;


// --------------------------------------------------------

class MapManager
{
	private :
		char *name;
		EBlockType ** map;
		int keysCount;
		GLPoint entryPoint;
		GLPoint enemyEntryPoint;
		GLPoint exitPoint;
		GLPoint size;

		char *path;
		char *nextMapName;
	public :
		MapManager(const char *mapsPath);
		virtual ~MapManager();

		void setPath(const char *mapsPath);
		void loadMap(const char *name);

		EBlockType getElement(int x,int y) const;
		EBlockType getElement(const GLPoint& position) const;
		void setElement(int x, int y, EBlockType element);
		void setElement(const GLPoint& position, EBlockType element);
		int getMapWidth(void) const;
		int getMapHeight(void) const;
		const GLPoint& getSize(void) const;
		GLPoint getPixelSize(void) const;

		const char *getNextMapName(void) const;
		const char *getName(void) const;

		bool isInMap(int x,int y) const;
		bool isInMap(const GLPoint& position) const;
		bool isScale(int x,int y) const;
		bool isScale(const GLPoint& position) const;
		bool isGround(int x,int y) const;
		bool isGround(const GLPoint& position) const;
		bool isBlank(int x,int y) const;
		bool isBlank(const GLPoint& position) const;
		bool isKey(int x,int y) const;
		bool isKey(const GLPoint& position) const;
		bool isExitPoint(int x, int y) const;
		bool isExitPoint(const GLPoint& position) const;

		void deleteItem(int x,int y);
		void deleteItem(const GLPoint& position);
		void getEntryPoint(int *x, int *y) const;
		const GLPoint& getEntryPoint(void) const;
		void getEnemyEntryPoint(int *x, int *y) const;
		const GLPoint& getEnemyEntryPoint(void) const;
		int getKeysCount(void) const;
};

// --------------------------------------------------------
// inline methods

inline
EBlockType MapManager::getElement(int x,int y) const
{
	if(isInMap(x,y))
		return map[y][x];
	else
		return btOutOfMap;
}

// --------------------------------------------------------

inline
EBlockType MapManager::getElement(const GLPoint& position) const
{
	return getElement(position.x, position.y);
}

// --------------------------------------------------------

inline
int MapManager::getMapWidth(void) const
{
	return size.x;
}

// --------------------------------------------------------

inline
int MapManager::getMapHeight(void) const
{
	return size.y;
}

// --------------------------------------------------------

inline
bool MapManager::isInMap(int x, int y) const
{
	return ((x >=0) && (x < getMapWidth()) && (y >= 0) && (y < getMapHeight()));
}

// --------------------------------------------------------

inline
bool MapManager::isInMap(const GLPoint& position) const
{
	return isInMap(position.x, position.y);
}

// --------------------------------------------------------

inline
bool MapManager::isScale(int x, int y) const
{
	return (getElement(x,y) == 's' || getElement(x,y) == 'u' || getElement(x,y) == 'd');
}

// --------------------------------------------------------

inline
bool MapManager::isScale(const GLPoint& position) const
{
	return isScale(position.x, position.y);
}

// --------------------------------------------------------

inline
bool MapManager::isGround(int x, int y) const
{
	return (getElement(x,y) == 'f');
}

// --------------------------------------------------------


inline
bool MapManager::isGround(const GLPoint& position) const
{
	return isGround(position.x, position.y);
}

// --------------------------------------------------------

inline
bool MapManager::isBlank(int x, int y) const
{
	return (getElement(x,y) == '#' || getElement(x,y) == 'k');
}

// --------------------------------------------------------

inline
bool MapManager::isBlank(const GLPoint& position) const
{
	return isBlank(position.x, position.y);
}

// --------------------------------------------------------

inline
bool MapManager::isKey(int x, int y) const
{
	return (getElement(x,y) == 'k');
}

// --------------------------------------------------------


inline
bool MapManager::isKey(const GLPoint& position) const
{
	return isKey(position.x, position.y);
}

// --------------------------------------------------------

inline
void MapManager::getEntryPoint(int *x, int *y) const
{
	*x = entryPoint.x;
	*y = entryPoint.y;
}

// --------------------------------------------------------

inline
const GLPoint& MapManager::getEntryPoint(void) const
{
	return entryPoint;
}

// --------------------------------------------------------

inline
void MapManager::getEnemyEntryPoint(int *x, int *y) const
{
	*x = enemyEntryPoint.x;
	*y = enemyEntryPoint.y;
}

// --------------------------------------------------------

inline
const GLPoint& MapManager::getEnemyEntryPoint(void) const
{
	return enemyEntryPoint;
}

// --------------------------------------------------------

inline
bool MapManager::isExitPoint(int x, int y) const
{
	return (x == exitPoint.x && y==exitPoint.y);
}

// --------------------------------------------------------


inline
bool MapManager::isExitPoint(const GLPoint& position) const
{
	return isExitPoint(position.x, position.y);
}

// --------------------------------------------------------

inline
int MapManager::getKeysCount(void) const
{
	return keysCount;
}

// --------------------------------------------------------

inline
const char *MapManager::getNextMapName(void) const
{
	return nextMapName;
}

// --------------------------------------------------------

inline
void MapManager::deleteItem(int x, int y)
{
	map[y][x] = btBlank;
}

// --------------------------------------------------------

inline
void MapManager::deleteItem(const GLPoint& position)
{
	deleteItem(position.x, position.y);
}

// --------------------------------------------------------

inline
const GLPoint& MapManager::getSize(void) const
{
	return size;
}

// --------------------------------------------------------

inline
GLPoint MapManager::getPixelSize(void) const
{
	return size * _MAP_BLOCK_SIZE;
}

// --------------------------------------------------------

inline
const char *MapManager::getName(void) const
{
	return name;
}

// --------------------------------------------------------

#endif // _MAPMANAGER_
